@@ -1,4 +1,3 @@
-from functools import partial
 import json
 import hashlib
 import os
@@ -7,21 +6,17 @@ import re
 import xml.etree.ElementTree as ET
 import zipfile
 
-from django.core.files.base import ContentFile
+from django.core.files import File
 from django.core.files.storage import default_storage
 from django.template import Context, Template
 from django.utils import timezone
-from django.utils.module_loading import import_string
 from webob import Response
 import pkg_resources
-from six import string_types
 
 from web_fragments.fragment import Fragment
 from xblock.core import XBlock
-from xblock.completable import CompletableXBlockMixin
 from xblock.fields import Scope, String, Float, Boolean, Dict, DateTime, Integer
 
-
 # Make '_' a no-op so we can scrape strings
 def _(text):
     return text
@@ -31,7 +26,7 @@ logger = logging.getLogger(__name__)
 
 
 @XBlock.wants("settings")
-class ScormXBlock(XBlock, CompletableXBlockMixin):
+class ScormXBlock(XBlock):
     """
     When a user uploads a Scorm package, the zip file is stored in:
 
@@ -48,26 +43,6 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
 
     Note that neither the folder the folder nor the package file are deleted when the
     xblock is removed.
-
-    By default, static assets are stored in the default Django storage backend. To
-    override this behaviour, you should define a custom storage function. This
-    function must take the xblock instance as its first and only argument. For instance,
-    you can store assets in different directories depending on the XBlock organisation with::
-
-        def scorm_storage(xblock):
-            from django.conf import settings
-            from django.core.files.storage import FileSystemStorage
-            from openedx.core.djangoapps.site_configuration.models import SiteConfiguration
-
-            subfolder = SiteConfiguration.get_value_for_org(
-                xblock.location.org, "SCORM_STORAGE_NAME", "default"
-            )
-            storage_location = os.path.join(settings.MEDIA_ROOT, subfolder)
-            return get_storage_class(settings.DEFAULT_FILE_STORAGE)(location=storage_location)
-
-        XBLOCK_SETTINGS["ScormXBlock"] = {
-            "STORAGE_FUNC": scorm_storage,
-        }
     """
 
     display_name = String(
@@ -118,13 +93,6 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
         scope=Scope.settings,
     )
 
-    fullscreen_on_launch = Boolean(
-        display_name=_("Fullscreen on launch"),
-        help=_("Display in fullscreen mode on launch"),
-        default=False,
-        scope=Scope.settings,
-    )
-
     has_author_view = True
 
     def render_template(self, template_path, context):
@@ -159,12 +127,7 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
         frag.add_css(self.resource_string("static/css/scormxblock.css"))
         frag.add_javascript(self.resource_string("static/js/src/scormxblock.js"))
         frag.initialize_js(
-            "ScormXBlock",
-            json_args={
-                "scorm_version": self.scorm_version,
-                "fullscreen_on_launch": self.fullscreen_on_launch,
-                "scorm_data": self.scorm_data,
-            },
+            "ScormXBlock", json_args={"scorm_version": self.scorm_version}
         )
         return frag
 
@@ -177,7 +140,6 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
             "field_weight": self.fields["weight"],
             "field_width": self.fields["width"],
             "field_height": self.fields["height"],
-            "field_fullscreen_on_launch": self.fields["fullscreen_on_launch"],
             "scorm_xblock": self,
         }
         studio_context.update(context or {})
@@ -190,17 +152,18 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
 
     @staticmethod
     def json_response(data):
-        return Response(json.dumps(data), content_type="application/json")
+        return Response(
+            json.dumps(data), content_type="application/json", charset="utf8"
+        )
 
     @XBlock.handler
     def studio_submit(self, request, _suffix):
         self.display_name = request.params["display_name"]
-        self.width = parse_int(request.params["width"], None)
-        self.height = parse_int(request.params["height"], None)
-        self.has_score = request.params["has_score"] == "1"
-        self.weight = parse_float(request.params["weight"], 1)
-        self.fullscreen_on_launch = request.params["fullscreen_on_launch"] == "1"
-        self.icon_class = "problem" if self.has_score else "video"
+        self.width = request.params["width"]
+        self.height = request.params["height"]
+        self.has_score = request.params["has_score"]
+        self.weight = request.params["weight"]
+        self.icon_class = "problem" if self.has_score == "True" else "video"
 
         response = {"result": "success", "errors": []}
         if not hasattr(request.params["file"], "file"):
@@ -210,85 +173,59 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
         package_file = request.params["file"].file
         self.update_package_meta(package_file)
 
-        # Clean storage folder, if it already exists
-        self.clean_storage()
+        # First, save scorm file in the storage for mobile clients
+        if default_storage.exists(self.package_path):
+            logger.info('Removing previously uploaded "%s"', self.package_path)
+            default_storage.delete(self.package_path)
+        default_storage.save(self.package_path, File(package_file))
+        logger.info('Scorm "%s" file stored at "%s"', package_file, self.package_path)
+
+        # Then, extract zip file
+        if default_storage.exists(self.extract_folder_base_path):
+            logger.info(
+                'Removing previously unzipped "%s"', self.extract_folder_base_path
+            )
+            recursive_delete(self.extract_folder_base_path)
+        with zipfile.ZipFile(package_file, "r") as scorm_zipfile:
+            for zipinfo in scorm_zipfile.infolist():
+                default_storage.save(
+                    os.path.join(self.extract_folder_path, zipinfo.filename),
+                    scorm_zipfile.open(zipinfo.filename),
+                )
 
-        # Extract zip file
         try:
-            self.extract_package(package_file)
             self.update_package_fields()
         except ScormError as e:
             response["errors"].append(e.args[0])
 
         return self.json_response(response)
 
-    def clean_storage(self):
-        if self.storage.exists(self.extract_folder_base_path):
-            logger.info(
-                'Removing previously unzipped "%s"', self.extract_folder_base_path
-            )
-            self.recursive_delete(self.extract_folder_base_path)
-
-    def recursive_delete(self, root):
-        """
-        Recursively delete the contents of a directory in the Django default storage.
-        Unfortunately, this will not delete empty folders, as the default FileSystemStorage
-        implementation does not allow it.
-        """
-        directories, files = self.storage.listdir(root)
-        for directory in directories:
-            self.recursive_delete(os.path.join(root, directory))
-        for f in files:
-            self.storage.delete(os.path.join(root, f))
-
-    def extract_package(self, package_file):
-        with zipfile.ZipFile(package_file, "r") as scorm_zipfile:
-            zipinfos = scorm_zipfile.infolist()
-            root_path = None
-            root_depth = -1
-            # Find root folder which contains imsmanifest.xml
-            for zipinfo in zipinfos:
-                if os.path.basename(zipinfo.filename) == "imsmanifest.xml":
-                    depth = len(os.path.split(zipinfo.filename))
-                    if depth < root_depth or root_depth < 0:
-                        root_path = os.path.dirname(zipinfo.filename)
-                        root_depth = depth
-
-            if root_path is None:
-                raise ScormError(
-                    "Could not find 'imsmanifest.xml' file in the scorm package"
-                )
-
-            for zipinfo in zipinfos:
-                # Extract only files that are below the root
-                if zipinfo.filename.startswith(root_path):
-                    # Do not unzip folders, only files. In Python 3.6 we will have access to
-                    # the is_dir() method to verify whether a ZipInfo object points to a
-                    # directory.
-                    # https://docs.python.org/3.6/library/zipfile.html#zipfile.ZipInfo.is_dir
-                    if not zipinfo.filename.endswith("/"):
-                        dest_path = os.path.join(
-                            self.extract_folder_path,
-                            os.path.relpath(zipinfo.filename, root_path),
-                        )
-                        self.storage.save(
-                            dest_path,
-                            ContentFile(scorm_zipfile.read(zipinfo.filename)),
-                        )
-
     @property
     def index_page_url(self):
         if not self.package_meta or not self.index_page_path:
             return ""
         folder = self.extract_folder_path
-        if self.storage.exists(
+        if default_storage.exists(
             os.path.join(self.extract_folder_base_path, self.index_page_path)
         ):
             # For backward-compatibility, we must handle the case when the xblock data
             # is stored in the base folder.
             folder = self.extract_folder_base_path
             logger.warning("Serving SCORM content from old-style path: %s", folder)
-        return self.storage.url(os.path.join(folder, self.index_page_path))
+        return default_storage.url(os.path.join(folder, self.index_page_path))
+
+    @property
+    def package_path(self):
+        """
+        Get file path of storage.
+        """
+        return (
+            "{loc.org}/{loc.course}/{loc.block_type}/{loc.block_id}/{sha1}{ext}"
+        ).format(
+            loc=self.location,
+            sha1=self.package_meta["sha1"],
+            ext=os.path.splitext(self.package_meta["name"])[1],
+        )
 
     @property
     def extract_folder_path(self):
@@ -307,9 +244,6 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
 
     @XBlock.json_handler
     def scorm_get_value(self, data, _suffix):
-        """
-        Here we get only the get_value events that were not filtered by the LMSGetValue js function.
-        """
         name = data.get("name")
         if name in ["cmi.core.lesson_status", "cmi.completion_status"]:
             return {"value": self.lesson_status}
@@ -323,45 +257,36 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
     def scorm_set_value(self, data, _suffix):
         context = {"result": "success"}
         name = data.get("name")
-        publish_grade = False
-        completion_percent = None
 
         if name in ["cmi.core.lesson_status", "cmi.completion_status"]:
             self.lesson_status = data.get("value")
-            if data.get("value") in [
+            if self.has_score and data.get("value") in [
                 "completed",
                 "failed",
                 "passed",
             ]:
-                completion_percent = 1
-                if self.has_score:
-                    publish_grade = True
+                self.publish_grade()
+                context.update({"lesson_score": self.lesson_score})
         elif name == "cmi.success_status":
             self.success_status = data.get("value")
             if self.has_score:
                 if self.success_status == "unknown":
                     self.lesson_score = 0
-                publish_grade = True
+                self.publish_grade()
+                context.update({"lesson_score": self.lesson_score})
         elif name in ["cmi.core.score.raw", "cmi.score.raw"] and self.has_score:
             self.lesson_score = float(data.get("value", 0)) / 100.0
-            publish_grade = True
-        else:
-            self.scorm_data[name] = data.get("value", "")
-
-        if completion_percent is not None:
-            self.emit_completion(completion_percent)
-        if publish_grade:
             self.publish_grade()
             context.update({"lesson_score": self.lesson_score})
+        else:
+            self.scorm_data[name] = data.get("value", "")
 
         context.update({"completion_status": self.get_completion_status()})
         return context
 
     def publish_grade(self):
         self.runtime.publish(
-            self,
-            "grade",
-            {"value": self.get_grade(), "max_value": self.weight},
+            self, "grade", {"value": self.get_grade(), "max_value": self.weight},
         )
 
     def get_grade(self):
@@ -398,64 +323,44 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
         """
         Update version and index page path fields.
         """
-        imsmanifest_path = self.find_file_path("imsmanifest.xml")
-        imsmanifest_file = self.storage.open(imsmanifest_path)
-        tree = ET.parse(imsmanifest_file)
-        imsmanifest_file.seek(0)
-        namespace = ""
-        for _, node in ET.iterparse(imsmanifest_file, events=["start-ns"]):
-            if node[0] == "":
-                namespace = node[1]
-                break
-        root = tree.getroot()
-
-        prefix = "{" + namespace + "}" if namespace else ""
-        resource = root.find(
-            "{prefix}resources/{prefix}resource[@href]".format(prefix=prefix)
-        )
-        schemaversion = root.find(
-            "{prefix}metadata/{prefix}schemaversion".format(prefix=prefix)
-        )
-
-        if resource is not None:
-            self.index_page_path = resource.get("href")
-        else:
-            self.index_page_path = self.find_relative_file_path("index.html")
-        if (schemaversion is not None) and (
-            re.match("^1.2$", schemaversion.text) is None
-        ):
-            self.scorm_version = "SCORM_2004"
-        else:
-            self.scorm_version = "SCORM_12"
-
-    def find_relative_file_path(self, filename):
-        return os.path.relpath(self.find_file_path(filename), self.extract_folder_path)
-
-    def find_file_path(self, filename):
-        """
-        Search recursively in the extracted folder for a given file. Path of the first
-        found file will be returned. Raise a ScormError if file cannot be found.
-        """
-        path = self.get_file_path(filename, self.extract_folder_path)
-        if path is None:
+        self.index_page_path = ""
+        imsmanifest_path = os.path.join(self.extract_folder_path, "imsmanifest.xml")
+        try:
+            imsmanifest_file = default_storage.open(imsmanifest_path)
+        except IOError:
             raise ScormError(
-                "Invalid package: could not find '{}' file".format(filename)
+                "Invalid package: could not find 'imsmanifest.xml' file at the root of the zip file"
             )
-        return path
+        else:
+            tree = ET.parse(imsmanifest_file)
+            imsmanifest_file.seek(0)
+            self.index_page_path = "index.html"
+            namespace = ""
+            for _, node in ET.iterparse(imsmanifest_file, events=["start-ns"]):
+                if node[0] == "":
+                    namespace = node[1]
+                    break
+            root = tree.getroot()
 
-    def get_file_path(self, filename, root):
-        """
-        Same as `find_file_path`, but don't raise error on file not found.
-        """
-        subfolders, files = self.storage.listdir(root)
-        for f in files:
-            if f == filename:
-                return os.path.join(root, filename)
-        for subfolder in subfolders:
-            path = self.get_file_path(filename, os.path.join(root, subfolder))
-            if path is not None:
-                return path
-        return None
+            if namespace:
+                resource = root.find(
+                    "{{{0}}}resources/{{{0}}}resource".format(namespace)
+                )
+                schemaversion = root.find(
+                    "{{{0}}}metadata/{{{0}}}schemaversion".format(namespace)
+                )
+            else:
+                resource = root.find("resources/resource")
+                schemaversion = root.find("metadata/schemaversion")
+
+            if resource:
+                self.index_page_path = resource.get("href")
+            if (schemaversion is not None) and (
+                re.match("^1.2$", schemaversion.text) is None
+            ):
+                self.scorm_version = "SCORM_2004"
+            else:
+                self.scorm_version = "SCORM_12"
 
     def get_completion_status(self):
         completion_status = self.lesson_status
@@ -469,7 +374,11 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
         accessible at a url with that also includes this name.
         """
         default_scorm_location = "scorm"
-        return self.xblock_settings.get("LOCATION", default_scorm_location)
+        settings_service = self.runtime.service(self, "settings")
+        if not settings_service:
+            return default_scorm_location
+        xblock_settings = settings_service.get_settings_bucket(self)
+        return xblock_settings.get("LOCATION", default_scorm_location)
 
     @staticmethod
     def get_sha1(file_descriptor):
@@ -478,7 +387,10 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
         """
         block_size = 8 * 1024
         sha1 = hashlib.sha1()
-        for block in iter(partial(file_descriptor.read, block_size), ""):
+        while True:
+            block = file_descriptor.read(block_size)
+            if not block:
+                break
             sha1.update(block)
         file_descriptor.seek(0)
         return sha1.hexdigest()
@@ -487,12 +399,11 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
         """
         Inform REST api clients about original file location and it's "freshness".
         Make sure to include `student_view_data=openedxscorm` to URL params in the request.
-
-        Note: we are not sure what this view is for and it might be removed in the future.
         """
         if self.index_page_url:
             return {
                 "last_modified": self.package_meta.get("last_updated", ""),
+                "scorm_data": default_storage.url(self.package_path),
                 "size": self.package_meta.get("size", 0),
                 "index_page": self.index_page_path,
             }
@@ -511,46 +422,18 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
             ),
         ]
 
-    @property
-    def storage(self):
-        """
-        Return the storage backend used to store the assets of this xblock. This is a cached property.
-        """
-        if not getattr(self, "_storage", None):
 
-            def get_default_storage(_xblock):
-                return default_storage
-
-            storage_func = self.xblock_settings.get("STORAGE_FUNC", get_default_storage)
-            if isinstance(storage_func, string_types):
-                storage_func = import_string(storage_func)
-            self._storage = storage_func(self)
-
-        return self._storage
-
-    @property
-    def xblock_settings(self):
-        """
-        Return a dict of settings associated to this XBlock.
-        """
-        settings_service = self.runtime.service(self, "settings") or {}
-        if not settings_service:
-            return {}
-        return settings_service.get_settings_bucket(self)
-
-
-def parse_int(value, default):
-    try:
-        return int(value)
-    except (TypeError, ValueError):
-        return default
-
-
-def parse_float(value, default):
-    try:
-        return float(value)
-    except (TypeError, ValueError):
-        return default
+def recursive_delete(root):
+    """
+    Recursively delete the contents of a directory in the Django default storage.
+    Unfortunately, this will not delete empty folders, as the default FileSystemStorage
+    implementation does not allow it.
+    """
+    directories, files = default_storage.listdir(root)
+    for directory in directories:
+        recursive_delete(os.path.join(root, directory))
+    for f in files:
+        default_storage.delete(os.path.join(root, f))
 
 
 class ScormError(Exception):
